/*
 *	q`vf[^XgA
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "sma02.h"
#include "ps2mc.h"

static int doIt(int base);

static int Mode = LPT_NORMAL;
static int Force = 0;

void Usage(const char *me)
{
	const char *ptr;
	
	ptr = strrchr(me, '\\');
	if (ptr == NULL) ptr = strrchr(me, ':');
	if (ptr == NULL) ptr = me;
	else ptr ++;
	
	fprintf(stderr, "%s Build:%s. Programmed by Hanimar.\n", ptr, __DATE__);
	fprintf(stderr, "Usage: %s [/b] [/f]\n", ptr);
	fputs("/b : Bidirectional mode.\n", stderr);
	fputs("/f : Force restoring.\n", stderr);
	exit(EXIT_FAILURE);
}

void argchk(int argc, char **argv)
{
	int i, j;
	
	for (i = 1 ; i < argc ; i ++) {
		if (argv[i][0] == '/' || argv[i][0] == '-') {
			for (j = 1 ; argv[i][j] != '\0' ; j ++) {
				switch (argv[i][j]) {
				case 'b':
				case 'B':
					Mode = LPT_BIDIRECTIONAL;
					break;
				case 'f':
				case 'F':
					Force = 1;
					break;
				default:
					Usage(argv[0]);
				}
			}
			continue;
		}
		Usage(argv[0]);
	}

	return;
}


int main(int argc, char **argv)
{
	int base;
	UCHAR status;
	int rc;
	int retry;
	
	argchk(argc, argv);
	
	base = LptGetPortAddr(1);	/* LPT1 = 378H ̂͂ */
	rc = Sma02Open(base, Mode);
	if (rc < 0) {
		Sma02Close(base);
		WaitMSec(200L);
		rc = Sma02Open(base, Mode);
		if (rc < 0) {
			fputs("Can not connect to SmartMedia via LPT1.\n", stderr);
			Sma02Close(base);
			return EXIT_FAILURE;
		}
	}
	
	/* 200 ~bقǑ҂܂BӖ͂ȂłǁB */
	WaitMSec(200L);
	Sma02Write1Byte(base, 1, 0x9c);	/* VCC ON + ݋ */
	WaitMSec(20L);

	/* fBAĂ邩ׂ܂ */
	for (retry = 100 ; retry ; retry --) {
		status = Sma02Read1Byte(base, 1);
		if ((status & 0x80) == 0) break;
		Sma02Reset(base);
	}
	if ((status & 0x04) == 0) {
		fprintf(stderr, "SmartMedia is not set.(%02Xh)\n", status);
		Sma02Write1Byte(base, 1, 0x18);	/* VCC OFF */
		Sma02Close(base);
		return EXIT_FAILURE;
	}
	if (!Force) {
		int c;
		
		fputs("Warning:All data in the SmartMedia will be lost.\n", stderr);
		fputs("Continue?[y/n]", stderr);
		fflush(stderr);
		for (;;) {
			while (!kbhit());
			c = getch();
			if (c == 'y' || c == 'Y') break;
			if (c == 'n' || c == 'N') {
				Sma02Close(base);
				return EXIT_FAILURE;
			}
		}
		fputc('\n', stderr);
	}
	
	rc = doIt(base);
	
	Sma02Close(base);
	return rc == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
}

static int doIt(int base)
{
	int block;
	int retry;
	
	for (block = 0 ; block < 1024 ; block ++) {
		printf("Block#%04d/1024\r", 1 + block);

		/* ubN */
		for (retry = 4 ; retry ; retry --) {
			if (Sma02EraseBlock(base, block << 4) == 0) break;
		}
		if (!retry) {
			fprintf(stderr, "\nSorry, failed to erase block#%d\n", block);
			return -1;
		}
	}
	return 0;
}
