/*
 *	q`vf[^XgA
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "sma02.h"
#include "ps2mc.h"

static int doIt(int base, FILE *fpout);

static const char *FileName = NULL;
static int Mode = LPT_NORMAL;
static int Force = 0;
static int CorrectEDC = 0;

void Usage(const char *me)
{
	const char *ptr;
	
	ptr = strrchr(me, '\\');
	if (ptr == NULL) ptr = strrchr(me, ':');
	if (ptr == NULL) ptr = me;
	else ptr ++;
	
	fprintf(stderr, "%s Build:%s. Programmed by Hanimar.\n", ptr, __DATE__);
	fprintf(stderr, "Usage: %s [/b] [/f] [/c] <filename>\n", ptr);
	fputs("/b : Bidirectional mode.\n", stderr);
	fputs("/f : Force restoring.\n", stderr);
	fputs("/c : Correct EDC.\n", stderr);
	exit(EXIT_FAILURE);
}

void argchk(int argc, char **argv)
{
	int i, j;
	int fileSpecified = 0;
	
	for (i = 1 ; i < argc ; i ++) {
		if (argv[i][0] == '/' || argv[i][0] == '-') {
			for (j = 1 ; argv[i][j] != '\0' ; j ++) {
				switch (argv[i][j]) {
				case 'b':
				case 'B':
					Mode = LPT_BIDIRECTIONAL;
					break;
				case 'f':
				case 'F':
					Force = 1;
					break;
				case 'c':
				case 'C':
					CorrectEDC = 1;
					break;
				default:
					Usage(argv[0]);
				}
			}
			continue;
		}
		if (fileSpecified) Usage(argv[0]);
		FileName = argv[i];
		fileSpecified ++;
	}
	if (FileName == NULL) Usage(argv[0]);
	
	return;
}


int main(int argc, char **argv)
{
	int base;
	UCHAR status;
	int rc;
	int retry;
	FILE *fpin = NULL;
	long fileSize;
	
	argchk(argc, argv);
	
	fpin = fopen(FileName, "rb");
	if (fpin == NULL) {
		fprintf(stderr, "Can not open file \"%s\".\n", FileName);
		return EXIT_FAILURE;
	}
	fseek(fpin, 0L, SEEK_END);
	fileSize = ftell(fpin);
	fseek(fpin, 0L, SEEK_SET);
	if (fileSize != 8650752L) {
		fputs("File size is not valid.(must be 8,650,752 bytes)\n", stderr);
		fclose(fpin);
		return EXIT_FAILURE;
	}
	
	base = LptGetPortAddr(1);	/* LPT1 = 378H ̂͂ */
	rc = Sma02Open(base, Mode);
	if (rc < 0) {
		Sma02Close(base);
		WaitMSec(200L);
		rc = Sma02Open(base, Mode);
		if (rc < 0) {
			fputs("Can not connect to SmartMedia via LPT1.\n", stderr);
			Sma02Close(base);
			fclose(fpin);
			return EXIT_FAILURE;
		}
	}
	
	/* 200 ~bقǑ҂܂BӖ͂ȂłǁB */
	WaitMSec(200L);
	Sma02Write1Byte(base, 1, 0x9c);	/* VCC ON + ݋ */
	WaitMSec(20L);

	/* fBAĂ邩ׂ܂ */
	for (retry = 100 ; retry ; retry --) {
		status = Sma02Read1Byte(base, 1);
		if ((status & 0x80) == 0) break;
		Sma02Reset(base);
	}
	if ((status & 0x04) == 0) {
		fprintf(stderr, "SmartMedia is not set.(%02Xh)\n", status);
		Sma02Write1Byte(base, 1, 0x18);	/* VCC OFF */
		Sma02Close(base);
		fclose(fpin);
		return EXIT_FAILURE;
	}
	if (!Force) {
		int c;
		
		fputs("Warning:All data in the SmartMedia will be lost.\n", stderr);
		fputs("Continue?[y/n]", stderr);
		fflush(stderr);
		for (;;) {
			while (!kbhit());
			c = getch();
			if (c == 'y' || c == 'Y') break;
			if (c == 'n' || c == 'N') {
				Sma02Close(base);
				fclose(fpin);
				return EXIT_FAILURE;
			}
		}
		fputc('\n', stderr);
	}
	
	rc = doIt(base, fpin);
	fclose(fpin);
	
	Sma02Close(base);
	return rc == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
}

static UCHAR Table[] = {
	0x00, 0x87, 0x96, 0x11, 0xa5, 0x22, 0x33, 0xb4,
	0xb4, 0x33, 0x22, 0xa5, 0x11, 0x96, 0x87, 0x00,
	0xc3, 0x44, 0x55, 0xd2, 0x66, 0xe1, 0xf0, 0x77,
	0x77, 0xf0, 0xe1, 0x66, 0xd2, 0x55, 0x44, 0xc3,
	0xd2, 0x55, 0x44, 0xc3, 0x77, 0xf0, 0xe1, 0x66,
	0x66, 0xe1, 0xf0, 0x77, 0xc3, 0x44, 0x55, 0xd2,
	0x11, 0x96, 0x87, 0x00, 0xb4, 0x33, 0x22, 0xa5,
	0xa5, 0x22, 0x33, 0xb4, 0x00, 0x87, 0x96, 0x11,
	0xe1, 0x66, 0x77, 0xf0, 0x44, 0xc3, 0xd2, 0x55,
	0x55, 0xd2, 0xc3, 0x44, 0xf0, 0x77, 0x66, 0xe1,
	0x22, 0xa5, 0xb4, 0x33, 0x87, 0x00, 0x11, 0x96,
	0x96, 0x11, 0x00, 0x87, 0x33, 0xb4, 0xa5, 0x22,
	0x33, 0xb4, 0xa5, 0x22, 0x96, 0x11, 0x00, 0x87,
	0x87, 0x00, 0x11, 0x96, 0x22, 0xa5, 0xb4, 0x33,
	0xf0, 0x77, 0x66, 0xe1, 0x55, 0xd2, 0xc3, 0x44,
	0x44, 0xc3, 0xd2, 0x55, 0xe1, 0x66, 0x77, 0xf0,
	0xf0, 0x77, 0x66, 0xe1, 0x55, 0xd2, 0xc3, 0x44,
	0x44, 0xc3, 0xd2, 0x55, 0xe1, 0x66, 0x77, 0xf0,
	0x33, 0xb4, 0xa5, 0x22, 0x96, 0x11, 0x00, 0x87,
	0x87, 0x00, 0x11, 0x96, 0x22, 0xa5, 0xb4, 0x33,
	0x22, 0xa5, 0xb4, 0x33, 0x87, 0x00, 0x11, 0x96,
	0x96, 0x11, 0x00, 0x87, 0x33, 0xb4, 0xa5, 0x22,
	0xe1, 0x66, 0x77, 0xf0, 0x44, 0xc3, 0xd2, 0x55,
	0x55, 0xd2, 0xc3, 0x44, 0xf0, 0x77, 0x66, 0xe1,
	0x11, 0x96, 0x87, 0x00, 0xb4, 0x33, 0x22, 0xa5,
	0xa5, 0x22, 0x33, 0xb4, 0x00, 0x87, 0x96, 0x11,
	0xd2, 0x55, 0x44, 0xc3, 0x77, 0xf0, 0xe1, 0x66,
	0x66, 0xe1, 0xf0, 0x77, 0xc3, 0x44, 0x55, 0xd2,
	0xc3, 0x44, 0x55, 0xd2, 0x66, 0xe1, 0xf0, 0x77,
	0x77, 0xf0, 0xe1, 0x66, 0xd2, 0x55, 0x44, 0xc3,
	0x00, 0x87, 0x96, 0x11, 0xa5, 0x22, 0x33, 0xb4,
	0xb4, 0x33, 0x22, 0xa5, 0x11, 0x96, 0x87, 0x00
};

static void calcEDC(UCHAR *edc, const UCHAR *data)
{
	int i, c;

	edc[0] = edc[1] = edc[2] = 0;

	for (i = 0 ; i < 0x80 ; i ++) {
		c = Table[data[i]];

		edc[0] ^= c;
		if (c & 0x80) {
			edc[1] ^= ~i;
			edc[2] ^= i;
		}
	}
	edc[0] = ~edc[0];
	edc[0] &= 0x77;

	edc[1] = ~edc[1];
	edc[1] &= 0x7f;

	edc[2] = ~edc[2];
	edc[2] &= 0x7f;

	return;
}

static int correctEDC(UCHAR *buffer)
{
	UCHAR buf[3];
	int i;
	UCHAR *data;
	UCHAR *edc;
	int ret = 0;
	
	for (i = 0 ; i < 4 ; i ++) {
		data = buffer + 0x80 * i;
		edc = buffer + 512 + 3 * i;
		
		calcEDC(buf, data);
		if ((edc[0] & 0x77) != buf[0] ||
			(edc[1] & 0x7f) != buf[1] ||
			(edc[2] & 0x7f) != buf[2])
		{
			edc[0] = buf[0];
			edc[1] = buf[1];
			edc[2] = buf[2];
			ret ++;
		}
	}
	
	return ret;
}

static int doIt(int base, FILE *fpin)
{
	int block;
	UCHAR buffer[528];
	int page;
	int retry;
	
	for (block = 0 ; block < 1024 ; block ++) {
		printf("Block#%04d/1024\r", 1 + block);

		/* ubN */
		for (retry = 4 ; retry ; retry --) {
			if (Sma02EraseBlock(base, block << 4) == 0) break;
		}
		if (!retry) {
			fprintf(stderr, "\nSorry, failed to erase block#%d\n", block);
			return -1;
		}
		for (page = 0 ; page < 16 ; page ++) {
			/* size_t  unsigned int  */
			if (fread(buffer, 1, 528, fpin) != 528) break;
			if (CorrectEDC) {
				if (correctEDC(buffer))
					fprintf(stderr, "\nCorrected EDC at block:%d page %d.\n",
						block + 1, page);
			}
			for (retry = 3 ; retry ; retry --) {
				if (Sma02WritePage(base, buffer, (block << 4) + page) == 0) break;
			}
			if (!retry) {
				fprintf(stderr, "\nSorry, failed to write page #%d\n", block * 16 + page);
				return -1;
			}
		}
	}
	return 0;
}
